#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
# pylint: disable=wrong-import-position
"""
GtkMe is a wrapper layer to make interacting with Gtk a little less painful.
The main issues with Gtk is that it expects an aweful lot of the developer,
code which is repeated over and over and patterns which every single developer
will use are not given easy to use convience functions.

This makes Gtk programming dry, unattractive and error prone. GtkMe steps in
between and adds in all those missing bits. It's not meant to replace Gtk and
certainly it's very possible to remove GtkMe completely and fall back on Gtk
and threading only. But better you than me fine sir.
"""

import threading
import os
import logging

import gi
gi.require_version('Gtk', '3.0')

from .window import Window, ChildWindow
from .forms import FormWindow
from .listview import TreeView, IconView, ViewColumn, ViewSort, Separator
from .pixmap import PixmapManager, AnimationManager
from .main import app, Thread, GtkApp, FakeWidget



#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
A special child window is called the FormWindow, it controls a GtkNotebook
as if each tab was a page in a multi-page form. All data in the form is
collected automatically and can even be verified using a data structural
varification module.

The form window requires specific glade widgets to exist with specific ids
unlike any other window. You must have a set of buttons:

 'next' with signal 'next_page' when clicked
 'previous' with signal 'previous_page' when clicked
 'apply' is optional with 'apply' when clicked

You must also create a GtkNotebook called 'pages' with any settings you like.

"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import FormWindow like any other window
from gtkme import FormWindow, GtkApp, TreeView


class AppFormWindow(FormWindow):
    """Normal window like any other"""
    name = 'formapp'

    # We need to force this form to be primary as it's normally
    # a ChildWindow and expects to be loaded by the main window
    primary = True

    # Each form has a set of fields which can be gathered together
    # and then validated, defining those fields allows us to skip
    # manually grabbing all the data with the get_data function.
    fields = \
    [ { # Page 1
        'age'      : { 'maxLength': 3, 'minLength': 1, 'type': 'integer' },
        'postcode' : { 'maxLength': 20, 'minLength': 0 }, # default type is string
        'gender'   : { 'maxLength': 40, 'minLength': 0, 'enumeration': ['-','M','F'] }, # XSD enum rules are awesome
    },{ # Page 2
        'password' : { 'maxLength': 200, 'minLength': 6, },
        'confirm'  : { 'maxLength': 200, 'minLength': 6, 'match': 'password' }, # XSD matching rules are cool
        'email'    : { 'maxLength': 200, 'minLength': 0, 'pattern' : '.+@.+\..+' }, # XSD regex rules are hot
    },{ # Page 3
        'sex'      : { 'type' : 'boolean' },
        'flux'     : { 'type' : 'boolean' },
        'wax'      : { 'type' : 'boolean' },
    } ]

    def load_widgets(self):
        # Glade doesn't handle it's own comboboxes with it's own liststore
        # that we define inside glade-3, basically I'm surprised glade manages
        # to put on it's shoes in the morning without exploading a passing goat.
        self.a = TreeView(self.widget('field_gender'), liststore=self.widget('genders'))

    def get_data(self):
        # We can manually gather together data here if needed.
        return FormWindow.get_data(self) # But we won't


class FormWindowApp(GtkApp):
    glade_dir = './'
    app_name = 'formapp'
    windows = [ AppFormWindow ]


def data_output(**data):
    """When windows finish they can call back and deliver any data
    that the window might have collected. In the cae of the Form
    Window that is all the data from the input elements."""
    print data


if __name__ == '__main__':
    try:
        app = FormWindowApp(start_loop=True, callback=data_output)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Creates some nice functions for indicators
"""

import os
import logging
import gi
from gi.repository import Gtk

try:
    gi.require_version('AppIndicator3', '0.1')
    from gi.repository.AppIndicator3 import (
        IndicatorStatus as Status, IndicatorCategory as Category,
        Indicator as RawIndicator
    )
    APPLICATION_STATUS = Category.APPLICATION_STATUS
except ImportError:
    try:
        from gi.repository.AppIndicator import Status, Category
        from gi.repository.AppIndicator import AppIndicator as RawIndicator
        APPLICATION_STATUS = Category.APPLICATIONSTATUS
    except ImportError:
        APPLICATION_STATUS = None
        logging.debug("Indicator is disabled, AppIndicator GIR missing.")
except ValueError:
    APPLICATION_STATUS = None
    # System doesn't support indicators


class Indicator(object):
    """Provides an inheritable indicator service, we would have
    inherited directly from RawIndicator, but GObject forbids it."""
    category = APPLICATION_STATUS
    desc = 'New Indicator'
    icon = 'gtk-edit'
    active_desc = 'Active New Indicator'
    active_icon = 'gtk-attention'
    primary = True
    name = 'indicator'

    def __init__(self, gapp, parent=None, debug=False, callback=None):
        self.gapp = gapp
        icon = debug and 'gtk-edit' or self.icon
        active_icon = debug and 'gtk-save' or self.active_icon
        self.raw = RawIndicator.new(self.desc, icon, self.category)
        self.raw.set_status(Status.ACTIVE)
        self.raw.set_attention_icon_full(active_icon, self.active_desc)
        # create a menu
        self.menu = Gtk.Menu()
        self.raw.set_menu(self.menu)
        self.wid = self.desc

    def init(self, *args, **kwargs):
        """Load widgets (do not override)"""
        self.load_widgets()

    def load_widgets(self):
        """Override this method to load your menus"""
        pass

    def new_menu(self, message, parent=None):
        """Create a new parent menuitem and attach a new menu to it"""
        if not parent:
            parent = self.menu
        item = self.new_subitem(message, parent, None)
        menu = Gtk.Menu()
        item.set_submenu(menu)
        return (item, menu)

    def new_item(self, message, method, *args):
        """Create a new menuitem and attach it to the indicator menu"""
        return self.new_subitem(message, self.menu, method, *args)

    def new_subitem(self, message, parent, method, *args):
        """Create a new menuitem, specify the parent menu object"""
        item = Gtk.MenuItem(message)
        item.show()
        parent.append(item)
        if method:
            item.connect("activate", method, *args)
        return item

    def new_separator(self, parent=None):
        """Starts a new section by creating a seperator"""
        if not parent:
            parent = self.menu
        item = Gtk.SeparatorMenuItem()
        item.show()
        parent.append(item)
        return item

    def deactivate(self):
        """Set indicator to passive"""
        self.raw.set_status(Status.PASSIVE)

    def activate(self, alert=False):
        """Set indicator to active"""
        if alert:
            self.raw.set_status(Status.ATTENTION)
        else:
            self.raw.set_status(Status.ACTIVE)

    def exit(self, widget=None):
        """Exit the indicator, destroy it"""
        self.gapp.remove_window(self)




#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Using the widgets in a window to do interesting things.
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme TreeView and ViewColumn class with the normal classes.
from gtkme import Window, GtkApp, PixmapManager
from listview import AppWindow as ListViewWindow


class HoldingWindow(Window):
    """A window to hold other widgets."""
    name = 'widgetextra'

    def load_widgets(self):
        target = self.widget('target')
        # Cloning is a way of allowing your designers to control all
        # the properties of elements which may need to be duplicated
        # Within the code.
        for x in range(2):
            widget = self.clone_widget('dupe', True)
            target.pack_start(widget, True, True, 0)
        # Adding a window as an element allows reuse of code and modualisation
        # of the design, can be useful for certain tab based applications.
        widget = self.load_window_extract('listapp')
        target.pack_start(widget, True, True, 5)
        widget.show_all()



class ListApp(GtkApp):
    glade_dir = './'
    app_name = 'extrawidgets'
    windows = [ HoldingWindow, ListViewWindow ]


if __name__ == '__main__':
    try:
        app = ListApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Wraps the gtk windows with something a little nicer.
"""
import logging
try:
    from xsdvalidate import Validator
except ImportError:
    Validator = None
from .main import FakeWidget, LoopDe
from gi.repository import Gtk, Gdk, GLib, GObject

PROPS = {
    'Box': ['expand', 'fill', 'padding', 'pack-type'],
    'Grid': ['top-attach', 'left-attach', 'height', 'width'],
    'Table': ['top-attach', 'left-attach', 'bottom-attach', 'right-attach'],
}

def protect(cls, *methods):
    """Simple check for protecting an inherrited class from having
    certain methods over-ridden"""
    if type(cls) != type:
        cls = type(cls)
    for method in methods:
        if method in cls.__dict__:
            raise RuntimeError("%s in %s has protected def %s()" \
                % (cls.__name__, cls.__module__, method))


class Window(object):
    """
    This wraps gtk windows and allows for having parent windows as well
    as callback events when done and optional quiting of the gtk stack.

    name = 'name-of-the-window'

    Should the window be the first loaded and end gtk when closed:

    primary = True/False

    A list of all the field elements in the window that should be given
    to the callback when exiting:

    fields = [ 'widget-name', ... ]

    """
    name    = None
    primary = True
    fields  = []

    def __init__(self, gapp, name=None):
        self.gapp = gapp
        self.name = name or self.name

        # Setup the gtk app connection
        self.w_tree = Gtk.Builder()
        self.widget = self.w_tree.get_object
        self.w_tree.set_translation_domain(gapp.app_name)
        self.w_tree.add_from_file(gapp.ui_file(self.name))

        # Setup the gtk builder window
        self.window = self.widget(self.name)
        if not self.window:
            raise Exception("Missing window widget '%s' from '%s'" % (
                self.name, gapp.ui_file(self.name)))

        # Give us a window id to track this window
        self.wid = str(hash(self.window))

    def extract(self):
        """Extract this window's container for use in other apps"""
        for child in self.window.get_children():
            self.window.remove(child)
            return child

    def init(self, parent=None, callback=None, **kwargs):
        if not 'replace' in kwargs:
            protect( self, 'destroy', 'exit', 'load_window', 'proto_window' )
        self.args     = kwargs
        # Set object defaults
        self.dead     = False
        self.parent   = parent
        self.callback = callback

        self.w_tree.connect_signals(self)

        # These are some generic convience signals
        self.window.connect('destroy', self.exit)

        # If we have a parent window, then we expect not to quit
        if self.parent:
            self.window.set_transient_for(self.parent)
            self.parent.set_sensitive(False)

        # We may have some more gtk widgets to setup
        self.load_widgets(**self.args)
        self.window.show()

    def __getattr__(self, name):
        """Catch signals with window names that need to be stripped."""
        # Be warned all ye who enter here, GTK will eat your errors and
        # exceptions and simply complain about missing signals. This happened
        # to me, don't let it happen to you!
        if self.name+'_' in name:
            function = name.replace(self.name+'_', '')
            return getattr(self, function)
        raise AttributeError("Window '%s' has no method or property '%s'" %(
            type(self).__name__, name))

    def load_window(self, name, *args, **kwargs):
        """Load child window, automatically sets parent"""
        kwargs['parent'] = self.window
        return self.gapp.load_window(name, *args, **kwargs)

    def load_window_extract(self, name, **kwargs):
        """Load a child window as a widget container"""
        window = self.gapp.proto_window(name)
        window.load_widgets(**kwargs)
        return window.extract()

    def load_widgets(self):
        """Child class should use this to create widgets"""
        pass

    def destroy(self, widget=None):
        """Destroy the window"""
        logging.debug("Destroying Window %s" % self.name)
        self.window.destroy()
        # We don't need to call self.exit(),
        # handeled by window event.

    def pre_exit(self):
        """Internal method for what to do when the window has died"""
        if self.callback:
            self.callback(self)

    def exit(self, widget=None):
        """Called when the window needs to exit."""
        if not widget or not isinstance(widget, Gtk.Window):
            self.destroy()
        # Clean up any required processes
        self.pre_exit()
        if self.parent:
            # We assume the parent didn't load another gtk loop
            self.parent.set_sensitive(True)
        # Exit our entire app if this is the primary window
        # Or just remove from parent window list, which may still exit.
        if self.primary:
            logging.debug("Exiting the application")
            self.gapp.exit()
        else:
            logging.debug("Removing Window %s from parent" % self.name)
            self.gapp.remove_window(self)
        # Now finish up what ever is left to do now the window is dead.
        self.dead = True
        self.post_exit()
        return widget

    def post_exit(self):
        """Called after we've killed the window"""
        pass

    def if_widget(self, name):
        """
        Attempt to get the widget from gtk, but if not return a fake that won't
        cause any trouble if we don't further check if it's real.
        """
        return self.widget(name) or FakeWidget(name)

    def clone_widget(self, widget, deep=False):
        """
        Attempt to create a clone of a widget named in the GtkBuilder. This is
        trying to use glade files as templates for sections of the code.
        """
        source = widget
        if isinstance(widget, str):
            source = self.widget(widget)
        dest = source.__class__()
        for setname in dir(source):
            if setname[:4] == "set_":
                value = None
                name = setname[4:]
                if name[:5] == 'from_':
                    name = name[5:]
                getname = 'get_' + name
                if name in ['buffer', 'data', 'parent']:
                    continue
                if hasattr(source, getname):
                    try:
                        value = getattr(source, getname)()
                    except TypeError:
                        continue
                elif hasattr(source, name):
                    value = getattr(source, name )
                if value == None:
                    continue
                if not isinstance(value, tuple):
                    value = tuple([value])
                try:
                    getattr(dest, setname)(*value)
                except TypeError:
                    continue
        if deep and isinstance(source, Gtk.Container):
            for child in source.get_children():
                self.clone_and_add(child, dest, True)
        return dest

    def clone_and_add(self, widget, container, deep=False):
        """Clones the widget and packs it into the container in the same way"""
        parent = widget.get_parent()
        nwidget = self.clone_widget(widget, deep=deep)
        container.add(nwidget)
        ctype = type(parent).__name__
        # Fix arangements and packing
        if ctype in PROPS.keys():
            for attach in PROPS[ctype]:
                # Introspection is very pokey.
                value = GObject.Value()
                value.init(GObject.TYPE_LONG)
                parent.child_get_property(widget, attach, value)
                container.child_set_property(nwidget, attach, value)

    def replace(self, old, new):
        """Replace the old widget with the new widget"""
        if isinstance(old, str):
            old = self.widget(old)
        if isinstance(new, str):
            new = self.widget(new)
        parent = old.get_parent()
        if parent is not None:
            parent.remove(old)
            parent.add(new)

    def double_click(self, widget, event):
        """This is the cope with gtk's rotten support for mouse events"""
        if event.type == Gdk.EventType._2BUTTON_PRESS:
            return self.apply(widget)


class ChildWindow(Window):
    """
    Base class for child window objects, these child windows are typically
    window objects in the same gtk builder file as their parents. If you just want
    to make a window that interacts with a parent window, use the normal
    Window class and call with the optional parent attribute.
    """
    primary = False



#!/usr/bin/python
#
# Copyright 2016 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Attempt to make a system log indicator (for those kind of people)
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme window and app class
from gtkme import GtkApp

from gtkme.indicator import Indicator


class LogIndicator(Indicator):
    desc = 'System Log Indicator'
    icon = "system-devices-panel"
    active_desc = "System has done things!"
    active_icon = "system-devices-panel-information"

    def load_widgets(self):
        # New items are added with their text and connected signal
        self.new_item("Alert Me!", self.signal)
        self.new_item("Passive Me!", self.signal2)
        # Seperators are created like so
        self.new_separator()
        # The exit signal is already expected and built in.
        self.new_item("Exit", self.exit)
        print "Indicator Ready!"

    def signal(self, widget=None):
        self.activate(True)

    def signal2(self, widget=None):
        self.activate(False)
        #self.deactivate()

class LogApp(GtkApp):
    windows = [LogIndicator]


if __name__ == '__main__':
    try:
        LogApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Interesting example using non-windows, lib-indicators with either
version of the standard from GIR will work.
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme window and app class
from gtkme import GtkApp

from gtkme.indicator import Indicator


class AppIndicator(Indicator):
    """Indicators follow similar rules to windows, they're components
    of the application that are always loaded like primary windows.
    Exiting the indicator will not exit the application unless it's the
    only 'window' running."""

    desc = 'Indicator Example'
    icon = "gtk-close"
    active_desc = "Example while activated"
    active_icon = "gtk-open"

    def load_widgets(self):
        # New items are added with their text and connected signal
        self.new_item("Alert Me!", self.signal)
        self.new_item("Passive Me!", self.signal2)
        # Seperators are created like so
        self.new_separator()
        # The exit signal is already expected and built in.
        self.new_item("Exit", self.exit)
        print "Indicator Ready!"

    def signal(self, widget=None):
        self.activate(True)

    def signal2(self, widget=None):
        self.deactivate()



class SimpleApp(GtkApp):
    """The application can also load indicators when required."""
    windows = [ AppIndicator ]


if __name__ == '__main__':
    try:
        app = SimpleApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/env python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
# Copyright 2015 Ian Denhardt <ian@zenhack.net>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Threaded windows allow your application to do tasks involving heavy
computaion or downloading which will interface with the gui at
intersections, but for which you need the interface to remain responsive.
"""

import threading
import logging
import time

# Import the threaded window class like others
from gtkme import asyncme, GtkApp, Window


class AppWindow(Window):
    """Just like any other window, except with threads."""
    # Use the existing app
    name = 'simpleapp'
    task_lock = threading.Lock()

    def sample_signal(self, widget=None):
        result = asyncme.holding(self.task_lock,
                               self.thread_contents,
                               blocking=False)
        if result is not None:
            print("Thread Started! (see, not locked)")

    def thread_contents(self):
        while True:
            timer = 0
            # This is where the action happens.
            while timer < 100:
                time.sleep(1)
                print("Ping")
                timer += 1
                self.safe_set_button_label('Foo %d' % timer)
                # TODO: We need some way of detecting if the gtk mainloop has
                # exited, and stop. The old ThreadedWindow code set a variable
                # on shutdown called  self._closed that we could check. I (Ian)
                # can think of a couple ways to do this, but I don't full
                # understand what gtkme is doing with the mainloop invocation
                # yet. For now, the application continues to run after the main
                # window has been closed (since this thread is still running).

    @asyncme.mainloop_only
    def safe_set_button_label(self, foo):
        # It isn't safe to use Gtk widgets outside of the thread executing
        # Gtk's mainloop, so this function is decorated with mainloop_only.
        # This forces it to be run inside of the Gtk mainloop.
        self.widget("button1").set_label(foo)


class ThreadedWindowApp(GtkApp):
    """This Application loads threaded windows just like any other."""
    glade_dir = './'
    app_name = 'threadedwindowapp'
    windows = [ AppWindow ]


if __name__ == '__main__':
    try:
        app = ThreadedWindowApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Sample example application using GtkMe
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme TreeView and ViewColumn class with the normal classes.
from gtkme import Window, GtkApp, TreeView, ViewColumn, ViewSort


class AppWindow(Window):
    """We need to load the window the list is in"""
    name = 'listapp'

    def load_widgets(self):
        self.mylist = TreeView(self.widget('listapp'))


class ListApp(GtkApp):
    glade_dir = './'
    app_name = 'listapp'
    windows = [ AppWindow ]


if __name__ == '__main__':
    try:
        app = ListApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Sample Application using pixmaps
"""

import os
import sys
import logging

sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Load a Pixmap Manager as well as the normal classes
from gtkme import Window, GtkApp, PixmapManager
from gtkme.pixmap import SizeFilter

image1 = 'raster' # PNG images
image2 = 'vector' # SVG images
image3 = 'gtk-close' # Gnome images
image4 = '\xff\xd8\xff\xe0\x00\x10JFIF\x00\x01\x01\x01\x00Z\x00Z\x00\x00\xff\xdb\x00C\x00\x0e\t\n\x0c\n\t\x0e\x0c\x0b\x0c\x0f\x0f\x0e\x10\x15"\x16\x15\x13\x13\x15*\x1e \x19"2,441,0/7>OC7:K;/0E^FKRTYYY5Bah`VgOWYU\xff\xdb\x00C\x01\x0f\x0f\x0f\x15\x12\x15(\x16\x16(U909UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU\xff\xc2\x00\x11\x08\x00\x0b\x00\x10\x03\x01\x11\x00\x02\x11\x01\x03\x11\x01\xff\xc4\x00\x17\x00\x00\x03\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x02\x03\x05\xff\xc4\x00\x17\x01\x01\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x03\x02\x00\x04\xff\xda\x00\x0c\x03\x01\x00\x02\x10\x03\x10\x00\x00\x01\xd5\x85JK\x97\x18\x1d\xff\xc4\x00\x19\x10\x01\x00\x03\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x01\x03\x12\x00\x04\xff\xda\x00\x08\x01\x01\x00\x01\x05\x02\xf4\xbe\xab[\xb6\xb2\x9c\x83\x0f\xff\xc4\x00\x1a\x11\x00\x02\x03\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x02\x03\x11\x12"\xff\xda\x00\x08\x01\x03\x01\x01?\x01\xa5\xc5?E\x9c\xe1$\x99\x87\xff\xc4\x00\x1d\x11\x00\x01\x04\x02\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x02\x03\x11\x12!"12\xff\xda\x00\x08\x01\x02\x01\x01?\x01l\xac.\xc6\xd7\x91\xcc\xed\x18\xdb})\xc5\x15\xff\xc4\x00\x1b\x10\x00\x02\x02\x03\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x02\x11!Aaq\xff\xda\x00\x08\x01\x01\x00\x06?\x02J/\xd3u\xd26\x8c#\xff\xc4\x00\x1a\x10\x01\x00\x03\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x11!1Aq\xff\xda\x00\x08\x01\x01\x00\x01?!J\xe5Z\xa3Q\x93\xa5t\xa6\xbb\xdb\xb2\xe4\x07\x93\xff\xda\x00\x0c\x03\x01\x00\x02\x00\x03\x00\x00\x00\x10\x95_\xff\xc4\x00\x19\x11\x00\x03\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x11!AQ\xff\xda\x00\x08\x01\x03\x01\x01?\x10\xf5\xaeQ\xf3r\xf0h\xa9\t\xe1\xff\xc4\x00\x1a\x11\x01\x00\x03\x00\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x11!A\xa1\xf0\xff\xda\x00\x08\x01\x02\x01\x01?\x10\xab5\xd4\x04\xc0S\x94T\xabyC\x04\xf6O\xff\xc4\x00\x1c\x10\x01\x01\x00\x02\x02\x03\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x11\x00!1aQq\xd1\xff\xda\x00\x08\x01\x01\x00\x01?\x10\xd2\xa5\x02\x8e\xb3\xc8q\xa6\xcfu\xc2\x80\r\xc9,:\xc5\x0fD\x9c&\x8f\x99\xff\xd9'

class TestImages(PixmapManager):
    # Where the images are located
    pixmap_dir = './images'
    # Global filters to apply to all images
    filters = [ SizeFilter ]


class PixmapWindow(Window):
    """Pixmap example window."""
    name = 'pixmapapp'

    def load_widgets(self):
        # Create a pixmap manager which controls where the images are
        # This can be made into an ineritied class where pixmap_dir is
        # set much like GtkApp and glade_dir or not.
        pix = TestImages('simple', size=128)

        # We can use the manager directly, or it can be used by lists
        # And other automatic features in gtkme.
        self.widget('image1').set_from_pixbuf(pix.get_pixmap(image1))
        self.widget('image2').set_from_pixbuf(pix.get_pixmap(image2))
        self.widget('image3').set_from_pixbuf(pix.get_pixmap(image3))
        self.widget('image4').set_from_pixbuf(pix.get_pixmap(image4))


class PixmapApp(GtkApp):
    """The application is in charge of loading the glade files, loads
    the windows"""
    glade_dir = './'
    app_name = 'pixmapapp'
    windows = [ PixmapWindow ]


if __name__ == '__main__':
    try:
        app = PixmapApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Special notebook based pages form child window for use with glade.
"""

import logging

from gi.repository import Gtk, Gdk, GLib, GObject

try:
    from validator import Validator
except ImportError:
    Validator = None

from .window import ChildWindow

class FormWindow(ChildWindow):
    """
    Base class from windows which act as forms, they expect to contain all
    kinds of fields and be able to validate them and pass the data back to
    the callback when required.
    """
    fields = None

    def __init__(self, *args, **kwrds):
        self.done      = False
        self._data     = {}
        self._marked   = {}
        self._pindex   = []
        super(FormWindow, self).__init__(*args, **kwrds)
        self._notebook = self.if_widget('pages')
        if self._notebook:
            self._notebook.connect("switch-page", self.update_buttons)
            for n in range(self._notebook.get_n_pages()):
                page = self._notebook.get_nth_page(n)
                self._pindex.append(page)

    def pre_exit(self):
        """Callback on exit with form data as args."""
        if self.callback and self.done:
            self.callback(**self.data())
            # Reset callback so super doesn't execute it.
            self.callback = None

    def data(self):
        """Access the data in the window."""
        # Refresh the data when the window is still alive
        # But don't when the window is closed (because we can't)
        if not self.dead:
            self._data = self.get_data()
        return self._data

    def get_data(self):
        """Return the data from all fields on the window."""
        if self.fields:
            return self.all_field_data()
        logging.warn("Can't get data, you need to define fields or create get_data function.")
        return None

    def next_page(self, widget=None):
        """When we have pages calling this event will move to the next"""
        working = self.is_valid(self._notebook.get_current_page())
        if working != True:
            return self.mark_invalid(working)
        # Call apply on this form if we've reached the end.
        if self._notebook.get_n_pages()-1 == self._notebook.get_current_page():
            return self.apply(widget)
        else:
            self._notebook.next_page()
        return widget

    def previous_page(self, widget=None):
        """when we have pages call this event and go back to the prev page"""
        notebook = self.if_widget('pages')
        notebook.prev_page()
        return widget

    def this_page(self, overide=None):
        """Return the current page we're on"""
        if overide == None:
            return self._notebook.get_current_page()
        return int(overide)

    @property
    def first_page(self):
        """Return the first page index, normally 0"""
        return 0

    @property
    def last_page(self):
        """Return the last page index, normally count -1"""
        return self._notebook.get_n_pages() - 1

    def update_buttons(self, widget, page_from, page_to):
        """Updates the previous, next and apply buttons"""
        this_page = self.this_page(page_to)
        # Enable the back button if the page is != 0
        self.update_button('previous', this_page > self.first_page)
        # Hide the forward and show the apply if the page is last
        if self.if_widget('apply'):
            self.update_button('apply', this_page == self.last_page)
            self.update_button('next', this_page < self.last_page)
        else:
            # Sometimes we hide the next button in glade so loading
            # screens can take the entire space. But now we need them.
            self.update_button('next', True)
        return this_page

    def update_button(self, name, enable):
        """Show/hide and/or enable/disable the named button"""
        #self.if_widget(name).set_sensitive(enable)
        self.if_widget(name).set_visible(enable)

    def set_page(self, pageid):
        """Set the form to a specific page"""
        if isinstance(pageid, int):
            self._notebook.set_current_page(pageid)
        elif isinstance(pageid, str):
            element = self.if_widget(pageid)
            if element and element in self._pindex:
                return self.set_page(self._pindex.index(element))
            else:
                raise KeyError("Couldn't find page element '%s'" % pageid)
        else:
            raise KeyError("Page id isn't an index or an element id.")

    def apply(self, widget=None):
        """Apply any changes as required by callbacks"""
        problems = self.is_valid()
        # True for is_valid or a list of problems
        if not problems or problems == True:
            logging.debug("Applying changes")
            self.done = True
            # Make sure arguments are stored.
            if not isinstance(self.data(), dict):
                logging.warn("%s data isn't a dictionary." % self.name)
            # Now exit the main application.
            self.destroy()
        else:
            self.mark_invalid(problems)
        return widget

    def is_valid(self, page=-1):
        "Return true if all data is valid or an array of invalid widget names." 
        # If fields is defined than we expect to do field validation.
        if self.fields:
            if isinstance(self.fields, list):
                # Checking single page
                if page >= 0 and page < len(self.fields):
                    return self.are_fields_valid(self.fields[page])
                else: # Checking all pages
                    for p in range(0, len(self.fields)):
                        errors = self.are_fields_valid(self.fields[p])
                        if errors != True:
                            return errors
            else: # No pages exist
                return self.are_fields_valid(self.fields)
        return True

    def are_fields_valid(self, f_def):
        """Returns true if the hash of field definitions are all valid."""
        if not f_def:
            return True
        if not Validator:
            logging.warn("Can't validate data: python-validator not installed.")
            return True
        data = self.field_data(f_def)
        errors = None
        field_list = []
        # Translate to xsd style syntax by transposing
        # the name into the hash from the key.
        for name in f_def.keys():
            field = f_def[name]
            field['name'] = name
            field['type'] = field.get('type', 'string')
            field_list.append( field )
        validator = Validator( { 'root' : field_list } )
#        try:
        errors = validator.validate( data )
#        except Exception, error:
#            logging.warn("Couldn't validate page, skipping validation: %s" % error)
        # Collect the errors and report on what fields failed
        if errors:
            result = []
            for err in errors.keys():
                if errors[err]:
                    result.append(err)
            return result
        # Otherwise we pass!
        return True

    def all_field_data(self):
        """Returns all of the fields we know about"""
        result = {}
        if isinstance(self.fields, list):
            for fields in self.fields:
                result.update(self.field_data(fields))
        return result

    def field_data(self, fields):
        """Return a simple hash of all the fields"""
        if not fields:
            return {}
        result = {}
        for name, field in fields.iteritems():
            widget = self.widget('field_' + name)
            if isinstance(widget, Gtk.TextView):
                buf = widget.get_buffer()
                start, end = ( buf.get_start_iter(), buf.get_end_iter() )
                result[name] = buf.get_text( start, end, True )
            elif isinstance(widget, Gtk.Entry):
                try:
                    result[name] = widget.get_text()
                except AttributeError:
                    result[name] = ''
            elif isinstance(widget, Gtk.ComboBox):
                # To avoid translations messing up the validation, with the
                # enumeration validation, this returns the index if no enum
                # is available, but will return the string if it is. Glade
                # is expected to translate, so don't translate your enum.
                active = widget.get_active()
                if active >= 0 and 'enumeration' in field:
                    active = field['enumeration'][active]
                result[name] = active
            elif isinstance(widget, Gtk.RadioButton):
                group = widget.get_group()
                if group:
                    for radio in group:
                        if radio.get_active():
                            result[name] = radio.get_label()
            elif isinstance(widget, Gtk.CheckButton) \
              or isinstance(widget, Gtk.Switch):
                result[name] = widget.get_active()
            elif widget:
                logging.warn("I can't understand '%s' fields for '%s'." % (str(type(widget)), name))
            else:
                logging.warn("Couldn't find field %s!" % name)
        return result

    def mark_invalid(self, invalids):
        """This is what happens when we're not valid"""
        if isinstance(self.fields, list):
            for f in self.fields:
                self._mark_invalid(invalids, f)
        else:
            self._mark_invalid(invalids, self.fields)

    def _mark_invalid(self, invalids, marks):
        """Only mark invalid on a set list"""
        if not marks:
            return
        for fname in marks.keys():
            widget = self.if_widget(fname+'_label')
            marked = self._marked.get(fname, None)
            if fname in invalids and not marked:
                self._marked[fname] = widget.get_label()
                widget.set_label(
                        "<span color='red'>*%s</span>" % self._marked[fname])
            elif fname not in invalids and marked:
                widget.set_label(self._marked.pop(fname))

    def cancel(self, widget=None):
        """We didn't like what we did, so don't apply"""
        self.done = False
        self.destroy()
        return widget

    




#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Base Classes etc.
"""
import threading
import os
import logging
import signal
from gi.repository import Gtk, GLib, GObject

class Thread(threading.Thread):
    """Special thread object for catching errors and logging them"""
    def run(self, *args, **kwargs):
        """The code to run when the thread us being run"""
        try:
            super(Thread, self).run(*args, **kwargs)
        except Exception as message: #W:0041
            logging.exception(message)


class LoopDe(object):
    """Base class for the looping interface"""
    def start(self):
        raise NotImplementedError("Loop interface needs a start() method.")

    def loop(self):
        raise NotImplementedError("Loop interface needs a loop() method.")

    def stop(self):
        raise NotImplementedError("Loop interface needs a stop() method.")

class Options(dict):
    """Simple dictionary of options and alternative static attribute defaults"""
    def __init__(self, obj, **dictionary):
        self.obj = obj
        self.update(dictionary)

    def __getitem__(self, name):
        if name in self:
            return dict.get(self, name)
        if hasattr(self.obj, name):
            return getattr(self.obj, name)
        raise KeyError("Can't find option: %s" % name)

    def __getattr__(self, name):
        return self[name]


class GtkApp(object):
    """
    This wraps gtk builder and allows for some extra functionality with
    windows, especially the management of gtk main loops.

    Options (class variable or passed in):

      app_name   - Name of application, for logging and translations.
      glade_dir  - Folder containing all glade files.
      prefix     - Folder prefix added to glade_dir.
      windows    - List of window classes attached to this App.
      start_loop - If set to true will start a new gtk main loop.

      **kwargs   - Passed to primary window when loaded.
    """
    @property
    def prefix(self):
        return self.opt.get('prefix', '')

    @property
    def windows(self):
        """Returns a list of windows for this app"""
        return self.opt.get('windows', None)

    @property
    def glade_dir(self):
        """This is often the local directory"""
        return self.opt.get('glade_dir', './')

    @property
    def app_name(self):
        """Set this variable in your class"""
        if self.opt.app_name:
            return self.opt.app_name
        raise NotImplementedError("App name is not set, pass in or set 'app_name' in class.")

    def __init__(self, **kwargs):
        self._loaded   = {}
        self._inital   = {}
        self._primary  = None
        self.main_loop = GLib.main_depth()
        self.opt       = Options(self, **kwargs)
        # Now start dishing out initalisation
        self.init_gui()
        # Start up a gtk main loop when requested
        if self.opt.start_loop:
            self.run_mainloop()

    def run_mainloop(self):
        """Run the gtk mainloop with ctrl+C and keyboard interupt additions"""
        logging.debug("Starting new GTK Main Loop.")
        # Add a signal to force quit on Ctrl+C (just like the old days)
        try:
            signal.signal(signal.SIGINT, signal.SIG_DFL)
            Gtk.main()
        except KeyboardInterrupt:
            logging.info("User Interputed")
        logging.debug("Exiting %s" % self.app_name)

    def ui_file(self, window):
        """Load any given gtk builder file from a standard location."""
        # Explaining Rant: For some reason it muddles the concepts of
        # application and window to such a degree that it's impossible
        # to create windows as seperated classes handeling their own events.
        path = os.path.join(self.glade_dir, self.opt.prefix, "%s.glade" % window)
        if not os.path.exists(path):
            path = os.path.join(self.glade_dir, self.opt.prefix, "%s.ui" % window)
            if not os.path.exists(path):
                raise Exception("Gtk Builder file is missing: %s" % path)
        return path

    def init_gui(self):
        """Initalise all of our windows and load their signals"""
        if self.opt.windows:
            for cls in self.opt.windows:
                window = cls
                logging.debug("Adding window %s to GtkApp\n" % window.name )
                self._inital[window.name] = window
            for window in self._inital.values():
                if window.primary:
                    if not self._primary:
                        self._primary = self.load_window(window.name)
                    else:
                        logging.debug("More than one window is set Primary!")
        if not self.opt.windows or not self._primary:
            logging.warn("No primary window found for '%s' app." % self.app_name)

    def load_window(self, name, *args, **kwargs):
        """Load a specific window from our group of windows"""
        window = self.proto_window(name)
        window.init(*args, **kwargs)
        return window

    def proto_window(self, name):
        """Loads a glade window as a window without initalisation, used for
        extracting widgets from windows without loading them as windows"""
        logging.debug("Loading '%s' from %s" % (name, str(self._inital)))
        if name in self._inital:
            # Create a new instance of this window
            window = self._inital[name](self)
            # Save the window object linked against the gtk window instance
            self._loaded[window.wid] = window
            return window
        raise KeyError("Can't load window '%s', class not found." % name)

    def remove_window(self, window):
        """Remove the window from the list and optional exit"""
        if window.wid in self._loaded:
            self._loaded.pop(window.wid)
        else:
            logging.warn("Tried unload window '%s' on exit, it's already gone."
                % window.name)
            logging.debug("Loaded windows: %s" % str(self._loaded))
        if not self._loaded:
            self.exit()

    def exit(self):
        """Exit our gtk application and kill gtk main if we have to"""
        if self.main_loop < GLib.main_depth():
            # Quit Gtk loop if we started one.
            logging.debug("Quit '%s' Main Loop." % (
                self._primary and self._primary.name or 'program'))
            Gtk.main_quit()
            # You have to return in order for the loop to exit
            return 0


class FakeWidget(object):
    """A fake widget class that can take calls"""
    def __init__(self, name):
        self._name = name

    def __getattr__(self, name):
        return self.fake_method

    def __bool__(self):
        return False

    def __nonzero__(self):
        return False

    def fake_method(self, *args, **kwargs):
        """Don't do anything, this is fake"""
        logging.info("Calling fake method: %s:%s" % (str(args), str(kwargs)))
        return None

    def information(self):
        """This is a dumb method too"""
        return None


class AutoApp(GtkApp):
    """Automatic application"""
    @property
    def glade_dir(self):
        return './'
    windows = [ ] #AppWindow ]


def app(*args, **kwargs):
    """An automatic application, loads the required everything"""
    kwargs['start_loop'] = True
    try:
        app = AutoApp(*args, **kwargs)
    except KeyboardInterrupt:
        logging.error("User Interputed")





#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Example showing trees of items in a list, seperators and comboboxes.
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme TreeView and ViewColumn class with the normal classes.
from gtkme import Window, GtkApp, TreeView, ViewColumn, Separator


class Item(object):
    """Example item to populate the list with"""
    def __init__(self, string, icon):
        self.string = string
        self.icon = icon


class ExampleTreeView(TreeView):
    """Show how easy it is to make a list of items with icons."""
    def get_markup(self, item):
        return item.string

    def get_icon(self, item):
        return item.icon

    def get_second(self, item):
        return item.second

    def setup(self, svlist):
        """Setup the treeview with one or many columns"""
        column = ViewColumn(svlist, 'Column Name', expand=True,
            text=self.get_markup, template="<b>%s</b>",
            icon=self.get_icon, pad=6, size=22 )



class AppWindow(Window):
    """We need to load the window the list is in"""
    name = 'treecombo'

    def load_widgets(self):
        # The same tree view can be used for both lists, trees and combo boxes
        self.mytree = ExampleTreeView(self.widget('treeview'), selected=self.signal)
        self.mydrop = ExampleTreeView(self.widget('combobox'), selected=self.signal)

        parent = self.mytree.add_item(Item('First Item', 'gtk-quit'))
        self.mytree.add_item(Item('Second Item', 'gtk-close'), parent)
        self.mytree.add_item(Separator())
        self.mytree.add_item(Item('Seperated Item', 'gtk-paste'))

        self.mydrop.add([
            Item('Third Item', 'gtk-open'),
            Separator(),
            Item('Fouth Item', 'find'),
            Item('Fith Item', 'gtk-cut'),
        ])
    
    def signal(self, item):
        print "Item selected! %s" % item.string



class ListApp(GtkApp):
    glade_dir = './'
    app_name = 'listapp'
    windows = [ AppWindow ]


if __name__ == '__main__':
    try:
        app = ListApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Example using an icon view instead of a list view.
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme TreeView and ViewColumn class with the normal classes.
from gtkme import Window, GtkApp, IconView


class Item(object):
    """Example item to populate the icons with"""
    def __init__(self, string, icon):
        self.string = string
        self.icon = icon


class ExampleIconView(IconView):
    """Show how easy it is to make a bunch of icons."""
    def get_markup(self, item):
        return item.string

    def get_icon(self, item):
        return item.icon


class IconViewWindow(Window):
    """We need to load the window the list is in"""
    name = 'iconviewapp'

    def load_widgets(self):
        self.mylist = ExampleIconView(self.widget('iconview'), selected=self.signal)
        self.mylist.add_item(Item('First Item', 'gtk-quit'))

        self.mylist.add([
            Item('Second Item', 'gtk-close'),
            Item('Third Item', 'gtk-open'),
            Item('Fouth Item', 'find'),
        ])
    
    def signal(self, item):
        print "Item selected! %s" % item.string



class IconViewApp(GtkApp):
    glade_dir = './'
    app_name = 'iconviewapp'
    windows = [ IconViewWindow ]


if __name__ == '__main__':
    try:
        app = IconViewApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Sample Application using interesting pixmaps
"""

import os
import sys
import logging

sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Load a Pixmap Manager as well as the normal classes
from gtkme import Window, GtkApp, PixmapManager
from gtkme.pixmap import SizeFilter, OverlayFilter, InternetPixmaps

image2 = 'image'
image1 = 'http://ubunchu.net/images/c1.png'
image3 = 'http://ubunchu.net/images/c2.png'
image4 = 'http://ubunchu.net/images/c3.png'


class TestImages(InternetPixmaps):
    # Where the images are located
    pixmap_dir = './images'
    # Global filters to apply to all images
    filters = [ SizeFilter, OverlayFilter ]


class PixmapWindow(Window):
    """Pixmap example window."""
    name = 'pixmapapp'

    def load_widgets(self):
        # Create a pixmap manager which controls where the images are
        # This can be made into an ineritied class where pixmap_dir is
        # set much like GtkApp and glade_dir or not.
        pix = TestImages('complex', size=128, overlay='overlay', placement=[0,1])

        # We can use the manager directly, or it can be used by lists
        # And other automatic features in gtkme.
        self.widget('image1').set_from_pixbuf(pix.get_pixmap(image1))
        self.widget('image2').set_from_pixbuf(pix.get_pixmap(image2))
        self.widget('image3').set_from_pixbuf(pix.get_pixmap(image3, overlay='other'))
        self.widget('image4').set_from_pixbuf(pix.get_pixmap(image4, overlay='other'))


class PixmapApp(GtkApp):
    """The application is in charge of loading the glade files, loads
    the windows"""
    glade_dir = './'
    app_name = 'pixmapapp'
    windows = [ PixmapWindow ]


if __name__ == '__main__':
    try:
        app = PixmapApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Wraps the gtk treeview and iconview in something a little nicer.
"""

import logging
from gi.repository import Gtk, Gdk, GObject, GdkPixbuf, Pango

from .pixmap import PixmapManager, SizeFilter

def default(item, attr, d=None):
    """Python logic to choose an attribute, call it if required and return"""
    if hasattr(item, attr):
        prop = getattr(item, attr)
        if callable(prop):
            prop = prop()
        return prop
    #logging.warning("No property '%s.%s'" % (type(item).__name__, attr))
    return d

def item_property(name, d=None):
    def inside(item):
        return default(item, name, d)
    return inside

def label(obj):
    if isinstance(obj, tuple):
        return ' or '.join([ label(o) for o in obj ])
    if not isinstance(obj, type):
        obj = type(obj)
    if hasattr(obj, '__name__'):
        return obj.__name__
    return str(obj)

class BaseView(object):
    """Controls for tree and icon views, a base class"""
    widget_type = None

    def __init__(self, widget, selected=None, unselected=None, liststore=None, **kwargs):
        if not isinstance(widget, self.widget_type):
            raise TypeError("Wrong widget type: Expected %s got %s" % ( label(self.widget_type), label(widget) ))
        if liststore is not None and not isinstance(liststore, Gtk.ListStore):
            raise TypeError("Wrong widget type: Expected ListStore got %s" % label(liststore) )
        elif liststore is None:
            liststore = widget.get_model()

        self.selected_signal = selected
        self.unselected_signal = unselected
        self._iids    = []
        self._list    = widget
        self.args     = kwargs
        self.selected = None
        self._model   = liststore
        self._data    = None
        self.no_dupes = True
        self.connect_signals()
        self.setup(self.remodel())
        super(BaseView, self).__init__()

    def connect_signals(self):
        """Try and connect signals to and from the view control"""
        signal = 'changed' # ComboBox
        if isinstance(self._list, Gtk.TreeView):
            signal = 'cursor_changed'
        elif isinstance(self._list, Gtk.IconView):
            signal = 'selection-changed'
        self._list.connect(signal, self.item_selected_signal)

    def remodel(self):
        """Setup the model and list"""
        return self._list

    def refresh(self):
        """Attempt to refresh the listview"""
        self._list.queue_draw()

    def setup(self, slist):
        """Setup columns, views, sorting etc"""
        pass

    def add(self, target, parent=None):
        """Add all items from the target to the treeview"""
        for item in target:
            self.add_item(item, parent=parent)

    def add_item(self, item, parent=None):
        """Add a single item image to the control"""
        if item is not None:
            iid = self.get_item_id(item)
            if iid:
                if iid in self._iids and self.no_dupes:
                    logging.debug("Ignoring item %s in list, duplicate." % iid)
                    return None
                self._iids.append(iid)
            return self._add_item(item, parent)
        else:
            raise Exception("Item can not be None.")

    def get_item_id(self, target):
        """Return if possible an id for this item,
           if set forces list to ignore duplicates,
           if returns None, any items added."""
        return target and None

    def replace(self, new_item, item_iter=None):
        """Replace all items, or a single item with object"""
        if item_iter:
            self.remove_item(target_iter=item_iter)
            self.add_item(new_item)
        else:
            self.clear()
            self._data = new_item
            self.add(new_item)

    def item_selected(self, item=None):
        """Base method result, called as an item is selected"""
        if self.selected != item:
            self.selected = item
            if self.selected_signal and item:
                self.selected_signal(item)
            elif self.unselected_signal and not item:
                self.unselected_signal(item)

    def remove_item(self, item=None, target_iter=None):
        """Remove an item from this view"""
        if target_iter and not item:
            return self._model.remove(target_iter)
        target_iter = self._model.get_iter_first()
        for itemc in self._model:
            if itemc[0] == item:
                return self._model.remove(target_iter)
            target_iter = self._model.iter_next(target_iter)

    def __iter__(self):
        target_iter = self._model.get_iter_first()
        for itemc in self._model:
            yield itemc[0], target_iter

    def set_sensitive(self, s):
        """Proxy the GTK property for sensitivity"""
        self._list.set_sensitive(s)

    def clear(self):
        """Clear all items from this treeview"""
        self._iids = []
        self._model.clear()

    def item_double_clicked(self, items):
        """What happens when you double click an item"""
        return items # Nothing

    def get_item(self, item_iter):
        """Return the object of attention from an iter"""
        if isinstance(item_iter, Gtk.TreePath):
            item_iter = self._model.get_iter(item_iter)
        return self._model[item_iter][0]

    def get_row(self, item):
        """Returns the iter for this item"""
        for i in self._model:
            if i[0] == item:
                return i
        return None

    def set_selected(self, item):
        """Sets the selected item to this item"""
        row = self.get_row(item)
        if row:
            self._list.set_cursor(row.path)


class TreeView(BaseView):
    """Controls and operates a tree view."""
    column_size = 16
    widget_type = (Gtk.TreeView, Gtk.ComboBox)

    def selected_items(self, treeview=None):
        """Return a list of selected item objects"""
        item_iter = None
        if not treeview:
            treeview = self._list
        # This may need more thought, only returns one item
        if isinstance(treeview, Gtk.TreeView):
            select = treeview.get_selection()
            if select:
                return [ self.safe_get_item(row) for row in select.get_selected_rows()[1] ]
        else:
            return [ self.safe_get_item( treeview.get_active_iter() ) ]

    def safe_get_item(self, item_iter):
        try:
            if item_iter != None:
                return self.get_item(item_iter)
        except TypeError as msg:
            logging.error("Error %s" % msg)
        logging.warn("No items selected for list?")
        return None

    def _add_item(self, item, parent):
        return self._model.append(parent, item)

    def item_selected_signal(self, treeview):
        """Signal for selecting an item"""
        items = self.selected_items(treeview)
        if items:
            return self.item_selected( items[0] )

    def item_button_clicked(self, treeview, event):
        """Signal for mouse button click"""
        if event.type == Gdk.EventType.BUTTON_PRESS:
            self.item_double_clicked( self.selected_items(treeview)[0] )

    def expand_item(self, item):
        """Expand one of our nodes"""
        self._list.expand_row(self._model.get_path(item), True)

    def remodel(self):
        """Set up an icon view for showing gallery images"""
        if not self._model:
            self._model = Gtk.TreeStore(GObject.TYPE_PYOBJECT)
        self._list.set_model(self._model)
        return self._list

    def ViewColumn(self, *args, **kwargs):
        return ViewColumn(self._list, *args, **kwargs)

    def ViewSort(self, *args, **kwargs):
        return ViewSort(self._list, *args, **kwargs)


class IconView(BaseView):
    """Allows a simpler IconView for DBus List Objects"""
    pixmaps = PixmapManager('', pixmap_dir='./', size=32)
    widget_type = Gtk.IconView

    def remodel(self):
        """Setup the icon view control and model"""
        self._model = Gtk.ListStore(GObject.TYPE_PYOBJECT, str, GdkPixbuf.Pixbuf)
        self._list.set_model(self._model)
        return self._list

    def setup(self, svlist):
        """Setup the columns for the iconview"""
        svlist.set_markup_column(1)
        svlist.set_pixbuf_column(2)

    def get_markup(self, item):
        """Default text return for markup."""
        return default(item, 'name', str(item))

    def get_icon(self, item):
        """Default icon return, pixbuf or gnome theme name"""
        return default(item, 'icon', None)

    def _get_icon(self, item):
        return self.pixmaps.get(self.get_icon(item))

    def _add_item(self, item, parent):
        # Each item's properties must be stuffed into the ListStore directly
        # or the IconView won't see them, but only if on auto.
        if not isinstance(item, (tuple, list)):
            item = [item, self.get_markup(item), self._get_icon(item)]
        return self._model.append(item)

    def item_selected_signal(self, icon_view):
        """Item has been selected"""
        return icon_view.get_selected_items()


class ViewSort(object):
    """Refute the use of sorting in gnome, bleh!"""
    def __init__(self, widget, data=None, ascending=False, exact=None, contains=None):
        self.tree = None
        self.data = data
        self.asc = ascending
        self.comp = exact.lower() if exact else None
        self.cont = contains
        self.tree = widget
        self.resort()

    def sort_func(self, model, iter1, iter2, data):
        value1 = self.data(model.get_value(iter1, 0))
        value2 = self.data(model.get_value(iter2, 0))
        if value1 == None or value2 == None:
            return 0
        if self.comp:
            if cmp(self.comp, value1.lower()) == 0:
                return -1
            elif cmp(self.comp, value2.lower()) == 0:
                return 1
            return 0
        elif self.cont:
            if self.cont in value1.lower():
                return -1
            elif self.cont in value2.lower():
                return 1
            return 0
        if value1 < value2:
            return 1
        if value2 < value1:
            return -1
        return 0

    def resort(self):
        model = self.tree.get_model()
        model.set_sort_func(0, self.sort_func, None)
        if self.asc:
            model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        else:
            model.set_sort_column_id(0, Gtk.SortType.DESCENDING)


class ViewColumn(object):
    """Add a column to a gtk treeview without dealing with gnomes"""
    def __init__(self, widget, name, expand=False, text=str, wrap=None, template=None,
        icon=False, pad=0, size=None, pixmaps=None, renderer=None):

        # Manager where icons will be pulled from
        self.pixmaps = pixmaps or PixmapManager('', pixmap_dir='./', size=size)
        self.size = SizeFilter(self.pixmaps, size=size)

        if isinstance(widget, Gtk.TreeView):
            column = Gtk.TreeViewColumn((name))
            column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
            column.set_expand(expand)
            widget.append_column(column)
        else:
            # Deal with possible drop down lists
            column = widget

        self._column = column

        # Separators should do something
        widget.set_row_separator_func(self.is_separator, None)

        if renderer is not None:
            renderer_custom = renderer()
            column.pack_start(renderer_custom, False)
            column.set_cell_data_func(renderer_custom, self.data_func(), None)

        if icon is not False:
            if icon is True or icon is None:
                icon = self.default_icon

            # The icon
            self._icon_renderer = \
            self.add_pix_renderer(
                func=self.icon_func(icon),
                pad=pad,
            )

        if text is not False:
            # The name
            self._text_renderer = \
            self.add_text_renderer(
                func=self.text_func(text or self.default_text, template),
                wrap=wrap,
            )

    def add_pix_renderer(self, func=None, pad=None, data=None):
        """Add a PixBuf renderer to the column"""
        renderer_icon = Gtk.CellRendererPixbuf()
        if pad is not None:
            renderer_icon.set_property("ypad", pad)
            renderer_icon.set_property("xpad", pad)
        self._column.pack_start(renderer_icon, False)

        if func is not None:
            self._column.set_cell_data_func(renderer_icon, func, data)

        return renderer_icon

    def add_text_renderer(self, func=None, wrap=None, data=None):
        """Add a CellRendererText to this column"""
        renderer = Gtk.CellRendererText()

        if wrap is not None:
            renderer.props.wrap_width = wrap
            renderer.props.wrap_mode = Pango.WrapMode.WORD

        self._column.pack_start(renderer, True)

        renderer.props.background_set = True
        renderer.props.foreground_set = True

        if func is not None:
            self._column.set_cell_data_func(renderer, func, data)

        return renderer

    def is_separator(self, model, item_iter, data):
        item = model.get_value(item_iter, 0)
        return isinstance(item, Separator)

    def clean(self, text, markup=False):
        """Clean text of any pango markup confusing chars"""
        if text == None:
            text = ''
        if isinstance(text, (str, int, float)):
            if markup:
                text = str(text).replace('<', '&lt;').replace('>', '&gt;')
            return str(text).replace('&', '&amp;')
        elif isinstance(text, dict):
            target = {}
            for key in text.keys():
                target[key] = self.clean(text[key])
            return target
        elif isinstance(text, list) or isinstance(text, tuple):
            target = []
            for value in text:
                target.append(self.clean(value))
            return tuple(target)
        else:
            raise TypeError("Unknown value type for text: %s" % str(type(text)))

    def get_callout(self, call, default=None):
        """Returns the right kind of method"""
        if isinstance(call, tuple):
            (call, default) = call
        if isinstance(call, str):
            call = item_property(call, default)
        return call

    def data_func(self):
        """Wrap a non-text data function"""
        def internal(column, cell, model, item_iter, data):
            item = model.get_value(item_iter, 0)
            cell.set_property("data", item)
        return internal

    def text_func(self, call, template=None):
        """Wrap up our text functionality"""
        callout = self.get_callout(call)
        def internal(column, cell, model, item_iter, data):
            if self.is_separator(model, item_iter, data):
                return
            item = model.get_value(item_iter, 0)
            markup = template is not None
            text = callout(item)
            if isinstance(template, str):
                text = template.format(self.clean(text, markup=True))
            else:
                text = self.clean(text)
            cell.set_property("markup", str(text))
            self._set_background(cell, item)
        return internal

    def icon_func(self, call):
        """Wrap, wrap wrap the func"""
        callout = self.get_callout(call)
        def internal(column, cell, model, item_iter, data):
            if self.is_separator(model, item_iter, data):
                return
            item = model.get_value(item_iter, 0)
            icon = callout(item)
            if (icon and isinstance(icon, str) and self.pixmaps) or icon is None:
                # Expect a Gnome theme icon
                icon = self.pixmaps.get(icon)
            elif icon:
                icon = self.size.filter(icon)
            if icon:
                cell.set_property("pixbuf", icon)
                cell.set_property("visible", True)
                self._set_background(cell, item)
            else:
                cell.set_property("visible", False)
        return internal

    def _set_background(self, cell, item):
        """Internal method for setting the background color"""
        color = self.set_background(item)
        if color:
            cell.set_property("cell-background", color)

    def set_background(self, item):
        """Returns None, over-ride and return color string i.e. '#ff0000'"""
        return None

    def default_text(self, item):
        """Default text return for markup."""
        return default(item, 'name', str(item))

    def default_icon(self, item):
        """Default icon return, pixbuf or gnome theme name"""
        return default(item, 'icon', None)


class Separator(object):
    """Reprisentation of a separator in a list"""
    pass



#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Sample example application using GtkMe
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme TreeView and ViewColumn class with the normal classes.
from gtkme import Window, GtkApp, TreeView, ViewColumn, ViewSort, Separator


class ExampleTreeView(TreeView):
    """Show how easy it is to make a list of items with icons."""
    def get_markup(self, item):
        return item._foo

    def setup(self, widget):
        """Setup the treeview with one or many columns manually"""
        # Data for text is pulled out of the string of the object (i.e. __unicode__)
        # If no icon is specified then no icon will apear. Default is off.
        self.ViewColumn('Column Name')

        # Option two, specify the attribute names to pull from using strings.
        # A tuple means the second value will be used as a default if no value.
        # market template and icon options can be added to tweak the display.
        self.ViewColumn('Second Column', expand=True,
            text='second', template="<b>%s</b>",
            icon=('icon', 'default-icon'), pad=0, size=12 )

        # Option three is completely manual, specifying a method to call to
        # Get the item's data, item is suplied as the only argument.
        self.ViewColumn('Third Column', expand=False,
            text=self.get_markup, template="<i>%s</i>",
            icon=lambda item: item.icon, pad=6, size=22)

        # Sort can sort by comparing column values, or give priority to terms.
        self.ViewSort(data=self.get_markup, ascending=True, contains='third')


class Item(object):
    """All List Items are Objects"""
    def __init__(self, name, second, icon):
        self._foo = name
        self.second = second
        self.icon = icon

    def __unicode__(self):
        return self._foo


class AppWindow(Window):
    """We need to load the window the list is in"""
    name = 'listapp'

    def load_widgets(self):
        self.a = self.load_listwidget(TreeView, 'automatic')
        self.b = self.load_listwidget(ExampleTreeView, 'manual')

    def load_listwidget(self, container, name):
        result = container(self.widget(name), selected=self.signal)

        # Add items one at a time
        result.add_item(Item('First Item', '1', 'gtk-quit'))

        # Add using an iterator many items
        result.add([
            Item('Second Item', '4', 'gtk-close'),
            Item('Third Item', '12', 'gtk-open'),
            Item('Fouth Item', '101011', 'find'),
        ])
        return result
    
    def signal(self, item):
        print "Item passed '%s' vs Item selected '%s'" % (unicode(item), unicode(self.a.selected))



class ListApp(GtkApp):
    glade_dir = './'
    app_name = 'listapp'
    windows = [ AppWindow ]


if __name__ == '__main__':
    try:
        app = ListApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Wraps the gtk pixmap access.
"""

import os
import logging

from gi.repository import Gtk, Gdk, GLib, GdkPixbuf

from .main import LoopDe

ICON_THEME = Gtk.IconTheme.get_default()
BILINEAR = GdkPixbuf.InterpType.BILINEAR
HYPER = GdkPixbuf.InterpType.HYPER


class AnimationManager(LoopDe):
    """Provides a way to animate an image widget
    (useful for please wait... screens)"""
    frame_from = 0
    frames = 1

    @property
    def pixmaps(self):
        raise NotImplementedError("No pixmaps definded for animation.")
    
    def __init__(self, widget):
        if isinstance(self.frames, int):
            self._frames = range(self.frame_from, self.frames)
        else:
            self._frames = self.frames
        self._length = len(self._frames)
        self._target = widget
        self._pos = 0

    def start(self):
        self._pos = self.frame_from
        self._target.show()
        self.loop()

    def loop(self):
        """Run once per thread turn"""
        self._pos += 1
        if self._pos >= self._length:
            self._pos = 0
        frame = str(self._frames[self._pos])
        image = self.pixmaps.get_pixmap(frame)
        self._target.set_from_pixbuf(image)

    def stop(self):
        """run once the thread is finished"""
        self._target.hide()


class PixmapFilter(object):
    """Base class for filtering the pixmaps in a manager's output.
       
       required - List of values required for this filter.

       Use:
       
       class Foo(PixmapManager):
           filters = [ PixmapFilterFoo ]

    """
    required = [ ]

    def __init__(self, manager, **kwargs):
        self.manager = manager
        missing = self.required[:]
        for key in kwargs.keys():
           if key in missing:
               missing.remove(key)
           setattr(self, key, kwargs[key])
        self.enabled = len(missing) == 0
        if not self.enabled:
            self.filter = self.do_nothing

    def do_nothing(self, img, **kwargs):
        """What disabled filters do"""
        return img

    def filter(self, img, **kwargs):
        """Run filter, replace this methodwith your own"""
        raise NotImplementedError("Please add 'filter' method to your PixmapFilter class %s." % type(self).__name__)



class OverlayFilter(PixmapFilter):
    """Adds an overlay to output images, overlay can be any name that
    the owning pixmap manager can find.

    overlay  : Name of overlay image
    location : Location of the image:
      0      - Full size (1 to 1 overlay, default)
      (x,y)  - Percentage from one end to the other position 0-1
    alpha    : Blending alpha, 0 - 255

    """
    overlay = None
    placement = 0
    alpha = 255

    def filter(self, img, overlay=None, **kwargs):
        overlay = overlay or self.overlay
        if overlay != None:
            overlay = self.manager.get(overlay, exempt=True)

            # Default values for full sized overlay
            width = img.get_width()
            height = img.get_height()
            (x, y) = (0, 0)

            if self.placement:
                (x, y, width, height) = self.set_position(overlay, width, height)

            if overlay:
                overlay.composite(img, x, y, width, height, x, y, 1, 1, BILINEAR, self.alpha)
            else:
                logging.warn("Couldn't find overlay '%s'" % self.overlay)
        return img

    def set_position(self, img, width, height):
        """Sets the position of img on the given width and height"""
        w = img.get_width()
        h = img.get_height()
        if w > width or h > height:
            logging.warn("Overlay is to large: %d > %d or %d > %d" % (w, width, h, height))
            return (0, 0, int(width), int(height))
        x = (width - w) * self.placement[0]
        y = (height - h) * self.placement[1]
        return (int(x), int(y), int(w), int(h))


class SizeFilter(PixmapFilter):
    """Resizes images to a certain size:
    
    resize_mode - Way in which the size is calculated
      0 - Best Aspect, don't grow
      1 - Best Aspect, grow
      2 - Cropped Aspect
      3 - Stretch
    """
    required = [ 'size' ]
    resize_mode = 0

    def __init__(self, *args, **kwargs):
        PixmapFilter.__init__(self, *args, **kwargs)
        if self.size:
            if isinstance(self.size, int):
                self.size = (self.size, self.size)
            if not hasattr(self.size, '__iter__') or len(self.size) < 2:
                raise AttributeError("Size needs to be a number or two number tuple.")
            self.w = float(self.size[0])
            self.h = float(self.size[1])

    def aspect(self, w, h):
        if self.resize_mode == 3 or (\
           self.resize_mode == 0 and w < self.w and h < self.h\
           ):
            return (w, h)
        (pw, ph) = (self.w / w, self.h / h)
        factor = self.resize_mode==2 and max(pw, ph) or min(pw, ph)
        return (int(w*factor), int(h*factor))

    def filter(self, img, **kwargs):
        if self.size != None:
            (width, height) = self.aspect(img.get_width(), img.get_height())
            return img.scale_simple(width, height, HYPER)
        return img


class PixmapManager(object):
    """Manage a set of cached pixmaps, returns the default image
    if it can't find one or the missing image if that's available."""
    missing_image = None
    default_image = 'default'
    icon_theme = ICON_THEME
    theme_size = 32
    filters = [ SizeFilter ]

    def get_pixmap_dir(self):
        """Set this variable in your class"""
        if not hasattr(self, 'pixmap_dir'):
            raise NotImplementedError("You need to set 'pixmap_dir' in the class.")
        return self.pixmap_dir

    def __init__(self, location, pixmap_dir=None, **kwargs):
        if pixmap_dir:
            self.pixmap_dir = pixmap_dir
        self.location = os.path.join(self.get_pixmap_dir(), location)
        self.loader_size = kwargs.pop('load_size', None)

        # Add any instance specified filters first
        self._filters = kwargs.get('filters', [])
        for lens in self.filters:
            # Now add any class specified filters with optional kwargs
            # Default: SizeFiler( size=required_field )
            self._filters.append(lens(self, **kwargs))

        self.cache    = {}
        self.get_pixmap(self.default_image)

    def get(self, *args, **kwargs):
        return self.get_pixmap(*args, **kwargs)

    def key_name(self, data):
        try:
            return (True, data) #.decode('UTF-8'))
        except UnicodeError:
            # Not a name or filename, take some bytes from the end
            try:
                return (False, data[-30:].encode('base64'))
            except UnicodeEncodeError:
                return (True, data)

    def get_pixmap(self, data, exempt=False, **kwargs):
        """Simple method for getting a set of pix pixmaps and caching them."""
        if not data:
            data = self.default_image
        (string, key) = self.key_name(data)

        if not key in self.cache:
            # load the image from data or a filename/theme icon
            if not string or '<svg' in data:
                self.cache[key] = self.load_from_data(data)
            else:
                self.cache[key] = self.load_from_name(data)

            # Filer the new image if not exempt from such things
            if self.cache[key] and not exempt:
                for lens in self._filters:
                    self.cache[key] = lens.filter(self.cache[key], **kwargs)

        if not key in self.cache or not self.cache[key]:
            key = self.default_image
        return self.cache.get(key, self.missing_image)

    def load_from_data(self, data):
        """Load in memory picture file (jpeg etc)"""
        # This doesn't work yet, returns None *shrug*
        loader = GdkPixbuf.PixbufLoader()
        if self.loader_size:
            loader.set_size(*self.loader_size)
        try:
            if isinstance(data, str):
                data = data.encode('utf-8')
            if not loader.write(data):
                raise TypeError(str("Failed to load."))
        except TypeError as err:
            logging.warning("Can't load pixbuf: {}".format(err))
            loader.close()
            return
        loader.close()
        return loader.get_pixbuf()

    def load_from_name(self, name):
        """Load a pixbuf from a name, filename or theme icon name"""
        pixmap_path = self.pixmap_path(name)
        if os.path.exists(pixmap_path):
            try:
                return GdkPixbuf.Pixbuf.new_from_file(pixmap_path)
            except RuntimeError as msg:
                logging.warn("No pixmap '%s', %s" % (pixmap_path, msg))
                return None 
        elif self.icon_theme and self.icon_theme.has_icon(name):
            return self.theme_pixmap(name, size=self.theme_size)
        # Nothing detectable
        if name != self.default_image:
            logging.warning("Can't find pixmap for %s in %s" % (name, self.location))
        return None

    def theme_pixmap(self, name, size=32):
        """Internal user: get image from gnome theme"""
        size = size or 32
        if not self.icon_theme.has_icon(name):
            name = 'image-missing'
        try:
            return self.icon_theme.load_icon(name, size, 0)
        except Exception as error:
            logging.debug("Can't load icon '%s': %s" % (name, str(error)))
            return None

    def pixmap_path(self, name):
        """Returns the pixmap path based on stored location"""
        svg_path = os.path.join(self.location, '%s.svg' % name)
        png_path = os.path.join(self.location, '%s.png' % name)
        if os.path.exists(name) and os.path.isfile(name):
            return name
        if os.path.exists(svg_path) and os.path.isfile(svg_path):
            return svg_path
        elif os.path.exists(png_path) and os.path.isfile(png_path):
            return png_path
        return os.path.join(self.location, name)


class InternetPixmaps(PixmapManager):
    """Provides access to internet images, chached to disk."""
    def __init__(self, *args, **kwargs):
        PixmapManager.__init__(self, *args, **kwargs)
        if not os.access(self.location, os.W_OK):
            raise IOError("Pixmap cache directory '%s' must be writable" % self.location)

    def url_name(self, url):
        return url.split('/')[-1]

    def pixmap_path(self, url):
        """Attempt to download and cache the requested file, return filename"""
        localname = PixmapManager.pixmap_path(self, url)
        if os.path.exists(localname):
            return localname
        filename = self.url_name(url)
        if '/' not in filename:
            filename = os.path.join(self.location, filename)
        if '.' not in filename:
            filename += '.image'
        if os.path.exists(filename):
            return filename
        if url[:7] != 'http://':
            return filename
        with open(filename, 'wb') as fhl:
            try:
                import urllib2
                webFile = urllib2.urlopen(url, None, 2)
                result = webFile.read()
            except Exception as error:
                logging.error("Error opening url '%s': %s" % (url, str(error)))
                return None
            webFile.close()
            fhl.write(result)
        return filename




#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Sample example application using GtkMe
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

# Import the gtkme window and app class
from gtkme import Window, GtkApp


class AppWindow(Window):
    """Each window has its own class, you can load multiple windows
    of the same type. This is a Primary window which means it is
    automatically shown when the app loads."""

    # Name of the window, this is both the filename of the glade
    # file 'simpleapp.glade' or 'simpleapp.ui' and also the id of
    # the window inside the glade file.
    name = 'simpleapp'

    # All signals are pre-bound, so you don't have to do anything
    # Except create the right definition in your python class.
    def sample_signal(self, widget=None):
        print "Button pressed!"

    # Exit/destroy signals are taken care of automatically.


class SimpleApp(GtkApp):
    """The application is in charge of loading the glade files, loads
    the windows"""
    # Where will the app load glade files from? This can be brought in
    # from a setup or configuration file for multiple platform deplyments.
    glade_dir = './'
    # Name of the application for logging.
    app_name = 'simpleapp'
    # A list of windows, windows that are primary windows will load
    # with the app, ChildWindows will load when requested using
    # app.load_window('window_name')
    windows = [ AppWindow ]


if __name__ == '__main__':
    # The App class will manage your gtk loop for you
    # Making sure to destroy it on exiting the last window.
    try:
        app = SimpleApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")








#
# Copyright 2011 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Wraps the gtk treeview and iconview in something a little nicer.
"""

import os
import mimetypes
import logging

from listview import TreeView, ViewColumn
from pixmap import ICON_THEME
from gi.repository import Gtk

MIME_CACHE = {}

class File(object):
    """Represent a file on the system"""
    def __init__(self, path, item=None):
        self.path = path
        self.item = item
        self.mime = mimetypes.guess_type(path)[0] or 'text/plain'
        if not hasattr(self, 'icon'):
            self.icon = self.get_icon()

    @property
    def name(self):
        return os.path.basename(self.path)

    @property
    def parent(self):
        return os.path.dirname(self.path)

    def get_icon(self):
        """Returns the right gnome icon"""
        global MIME_CACHE
        if not self.mime in MIME_CACHE:
            for possible in ( self.mime,
                              self.mime.replace('/', '-'),
                              self.mime.replace('/', '-x-'),
                              'text-plain',
                            ):
                if ICON_THEME.has_icon(possible):
                    MIME_CACHE[self.mime] = possible
                    break
        return MIME_CACHE[self.mime]

    def isfile(self):
        return True

    def ishidden(self):
        return self.name[0] == '.'

    def __str__(self):
        return str(self.path)

    def __repr__(self):
        return "<%s '%s'>" % (type(self).__name__, self.path)


class Directory(File):
    """Represent a dir on the system"""
    @property
    def icon(self):
        return 'gtk-directory'

    def isfile(self):
        return False

class FileTreeView(TreeView):
    """Load a treeview as a look at directories"""
    sorting = Gtk.SortType.ASCENDING
    file_class = File
    dir_class = Directory
    column_name = 'File Listing'
    column_size = 16

    def __init__(self, *args, **kwargs):
        self.show_all = kwargs.pop('show_all', False)
        kwargs['text'] = 'name'
        TreeView.__init__(self, *args, **kwargs)
        self.path = None

    def change_directory(self, path):
        """Change the current directory"""
        if not os.path.exists(path):
            raise IOError("Can't view path '%s' doesn't exist." % path)
        self.path = path
        self.update()

    def walk(self):
        """Over-ride this method if you want to construct file trees"""
        return os.walk(self.path, topdown=True)

    def clear(self):
        """Clear and disable"""
        TreeView.clear(self)
        self.set_sensitive(False)

    def filter(self, item):
        """Filter an item (over-ride to make use)"""
        return True

    def update(self):
        """Refresh the contents of the tree view"""
        self.clear()
        parents = {}
        for root, dirs, files in self.walk():
            if root == self.path or root == '':
                parent = None
            elif root in parents and parents[root]:
                parent = parents[root]
            else:
                print "Skipping %s: %s" % (root, parents.keys())
                # parent skipped, so child skipped too
                continue

            for name in dirs:
                path = os.path.join(root, name)
                parents[path] = self.add_item(self.dir_class(path, self), parent)

            for name in files:
                self.add_item(self.file_class(os.path.join(root, name), self), parent)

        self.set_sensitive(True)


    def add_item(self, item, parent=None):
        if (not item.ishidden() or self.show_all) and self.filter(item):
            return TreeView.add_item(self, item, parent)




#!/usr/bin/python
#
# Copyright 2012 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Use of the file listing functionality in a tree view.

The tree view is more of an extended example of how to use the tree view
models, but is also useful for anyone who wants to display lists of files.
"""

import os
import sys
import logging

# Directories where gtkme can be found locally
sys.path.insert(1, '../../lib')
sys.path.insert(1, './lib')

from gtkme import Window, GtkApp

# Import the file view
from gtkme.listfiles import FileTreeView


class ListFilesWindow(Window):
    """Show files in our list view, we can reuse the listapp glade file too."""
    name = 'listapp'

    def load_widgets(self):
        self.myfiles = FileTreeView(self.widget('manual'), selected=self.signal)
        self.myfiles.change_directory('/var/log/')
    
    def signal(self, item):
        print "Item selected! %s" % unicode(item)



class ListApp(GtkApp):
    glade_dir = './'
    app_name = 'listfilesapp'
    windows = [ ListFilesWindow ]


if __name__ == '__main__':
    try:
        app = ListApp(start_loop=True)
    except KeyboardInterrupt:
        logging.info("User Interputed")
    logging.debug("Exiting Application")





